from __future__ import division
import game
from math import *
from body import Body
from tile import Tile
from vector import Vector
from sprite import Sprite
from spark import Spark
from shot import Shot
import random, copy

import OpenGL
from OpenGL.GL import *

class Allefant(Sprite):
    def __init__(self, id, type, x, y, g_z):
        Sprite.__init__(self, id, type, x, y, g_z)
        self.rope = []
        attach = self.attach_point()
        for i in range(5):
            self.rope.append(attach + Vector(0, i * -8, 0))
        self.k = 0
        self.energy = True
        self.xm, self.ym = 0, 0
        self.turn(0, -0.1, 0)
        self.shoot = 0
        self.reason = ""
        self.off = None
        self.chok = 0

    def attach_point(self):
        return self.position + self.b * 4 * sqrt(2)
    
    def auto_level(self):
        if self.b.y > 0.9: return
        up = Vector(0, 1, 0)
        c = up.dot(self.r)
        a = acos(c)
        if a < pi / 2: self.turn(0, 0, -0.001)
        elif a > pi / 2: self.turn(0, 0, 0.001)

    def update(self):
        # "physics"
        self.tick()

        x, y, z = self.position.xyz()
        self.tx = int(x // Tile.w)
        self.tz = int(z // Tile.h)
        
        if y < -Tile.d * 10 or self.tx == 0 or self.tz == 0 or\
            self.tx == self.map.w - 1 or self.tz == self.map.h - 1:
            self.dead = True
            self.reason = "lost"
            return

        if self.off and self.off.position.y > Tile.d * 10:
            self.dead = True
            self.reason = "fell"
            return
        
        if self.shoot:
            self.shoot -= 1
        if self.chok:
            self.chok -= 1
        
        # steering
        map = self.map
        ax = 0.0
        ay = 0.0
        accel = 0.0
        RF = 0.02
        F = 0.01
        RF2 = RF * 2
        if self.k & 1: ax -= RF
        if self.k & 2: ax += RF
        if self.k & 4: ay += RF
        if self.k & 8: ay -= RF
        if self.k & 16:
            if self.energy:
                accel += F
                if self.chok == 0:
                    game.game.s_chok.play(0.5)
                    self.chok = 10
        else:
            self.chok = 0
        if self.k & 32:
            if self.shoot == 0 and not self.off:
                Shot(self.position - self.b * 20 + self.u * 20, self.momentum
                    - self.b * 2)
                self.momentum += self.b * 0.05
                self.shoot = 10
                game.game.s_shot.play(1)

        if self.off:
            accel = 0
            def r():
                return (random.random() - 0.5) * 0.05
            self.off.turn(r(), r(), r())
            self.off.force = Vector(0, 0.01, 0)
            self.off.tick()

        mmx = self.xm * RF
        mmy = self.ym * RF * -1
        if abs(mmx) > RF2: mmx = RF2 * mmx / abs(mmx)
        if abs(mmy) > RF2: mmy = RF2 * mmy / abs(mmy)
        ax += mmx
        ay += mmy
        
        down = -game.GRAVITY
        
        # handle air drag
        tile, h = self.map.get(self.tx, self.tz)
        if tile == 0:
            down -= game.GRAVITY * 3
            self.force += self.momentum * 0.01
        else:
            h = self.map.height_above_ground(x, z)
            if y < h + 16:
                d = h + 16 - y
                if d > 16: d = 16
                down += d / 16.0 * game.GRAVITY * 2
        self.force += Vector(0, down, 0)
        self.move(0, 0, -accel)
        self.turn(ay / 2 / pi, -ax / 2 / pi, 0)

        # handle collision (really broken)
        collision_points = [
            self.position, # bottom
            self.position - self.b * 30 + self.u * 20, # nose
            self.position + self.b * 10 + self.u * 20, # tail
            ]
            
        first = True
        for cpos in collision_points:
            h = self.map.height_above_ground(cpos.x, cpos.z)
            v = self.velocity_at(cpos - self.position)
            
            if cpos.y < h + 4:
                # um, well, just assuming things..
                if cpos.y < h - 4 or not first:
                    # we should apply a force at the collision point,
                    # but didn't figure out the formula yet
                    self.momentum = v * self.mass * -0.125
                    
                    if tile == 0:
                        self.momentum.y -= self.mass * 0.25
                    else:
                        # we add some upwards motion on any collision
                        self.momentum.y += self.mass * 0.25
                    # uh oh. now we're really fudging things
                    self.position += self.momentum * 2
                else:
                    # assume the collision normal is this.. shouldn't be hard
                    # getting the actual normal :P
                    n = Vector(0, 1, 0)
                    # blah, no. this is just wrong.
                    self.momentum = v.reflect(n) * self.mass
                    # get us out of the collision
                    self.position.y += (h + 4 - cpos.y) * 0.1
            
            first = False
        
        # handle attached string
        sparks = []
        pos = self.attach_point()
        for i in range(len(self.rope)):
            # try to maintain length
            self.rope[i].y -= 1
            v = self.rope[i] - pos
            n = v.norm()
            if n > 8:
                self.rope[i] = pos + v * (8 / n)
            
            # get us out of the ground
            x, y, z = self.rope[i].xyz()
            h = self.map.height_above_ground(x, z)
            if y < h + 4:
                self.rope[i].y = h + 4
                sparks.append((x, h + 4, z))

            pos = self.rope[i]
        
        if sparks:
            self.energy = True
            if random.random() < 0.1:
                x, y, z = random.choice(sparks)
                Spark(x, y, z)
                game.game.s_spark.play(0.3)
        else:
            self.energy = False
        
        self.auto_level()
        
        if self.u.y < -0.2 and not self.off:
            self.off = Body()
            self.off.position = copy.copy(self.position)
            self.off.orientation = copy.copy(self.orientation)
            self.off.tick()

        self.damp()

    def draw_3d(self):
        glPushMatrix()
        glMultMatrixf(self.matrix())
        if not self.off: game.game.meshes["Player"].render()
        game.game.meshes["Boat"].render()
        glPopMatrix()
        
        if self.off:
            glPushMatrix()
            glMultMatrixf(self.off.matrix())
            game.game.meshes["Player"].render()
            glPopMatrix()

        i = 0
        pos = self.attach_point()
        for rope in self.rope:
            glColor4f(1, 0.5 * i / 10.0, 0, 0.8)
            v, n = Tile.rope(pos, rope)
            pos = rope
            glNormalPointerf(n)
            glVertexPointerf(v)
            glDrawArrays(GL_QUADS, 0, len(v))

    def draw_2d(self):
        x, y, z = self.position.xyz()
        pic = game.game.boat
        h = self.map.height_above_ground(x, z)
        glColor4f(0, 0, 0, 0.5)
        pic.draw_3d2(x, h + 4, z)
        glColor4f(1, 1, 1, 1)