from __future__ import division

import pygame, sys, os, random

import OpenGL
from OpenGL.GL import *

from math import *
from image import Image
from text import Text, Font
from sound import Sound
from map import Map
from tachyon import Tachyon
from mesh import Mesh, load_meshes
from allefant import Allefant
from tile import Tile
import story

PLAYER = 1
TACHYON = 2
SPARK = 3
RAMPR = 4
TILE = 5
SHOT = 6
RAMPL = 8

GRAVITY = 0.004

class Game:
    FPS = 60
    size = 640, 480

    def init(self):
        self.w, self.h = self.size
        self.font = Font("DejaVuSans.ttf", 32)
        self.meshes = load_meshes("mesh.mm",
            lambda x, y, z: (x * 16, y * 16, z * 16))
        
        self.tachyon = []
        for i in range(16):
            self.tachyon.append(Image("string/tachyon%04d.png" % (1 + i)))

        self.spark = Image("string/spark.png")
        self.shot = Image("string/shot.png")
        self.boat = Image("string/boat.png")
        
        self.s_bling = Sound("sfxr_bling.wav")
        self.s_gone = Sound("sfxr_gone.wav")
        self.s_shot = Sound("sfxr_shot.wav")
        self.s_chok = Sound("sfxr_chok.wav")
        self.s_spark = Sound("sfxr_spark.wav")

        self.k = 0
        self.tick = 0
        self.mousepos = 0, 0
        
        self.tile = 5
        self.height = 0
        
        self.sprites = []
        
        self.map = Map()
        self.map.id = 1
        self.player = None
        self.editor = False
        self.message = ""
        self.message_time = 0
        
        self.load()

    def run(self):
        w, h = self.size
        quit = False
        next = pygame.time.get_ticks() + 1000 / self.FPS
        pygame.mouse.get_rel()
        while not quit:
        
            self.mousepos = pygame.mouse.get_pos()
            
            for e in pygame.event.get():
                if e.type == pygame.QUIT:
                    quit = True
                elif e.type == pygame.KEYDOWN:
                    if e.key == pygame.K_ESCAPE:
                        quit = True
                    if e.key == pygame.K_F10:
                        pygame.event.set_grab(not pygame.event.get_grab())
                    #if e.key == pygame.K_TAB or e.key == pygame.K_RETURN:
                    #    self.editor = not self.editor
                    if e.key == pygame.K_LEFT: self.k |= 1
                    if e.key == pygame.K_RIGHT: self.k |= 2
                    if e.key == pygame.K_UP: self.k |= 4
                    if e.key == pygame.K_DOWN: self.k |= 8
                    if e.key == pygame.K_z or e.key == pygame.K_y:
                        self.k |= 32
                    if e.key == pygame.K_a:
                        self.k |= 16
                        if not self.player and not self.editor:
                            self.load()
                    #if e.key == pygame.K_PAGEDOWN:
                    #    if self.tile > 0: self.tile -= 1
                    #if e.key == pygame.K_PAGEUP:
                    #    if self.tile < 16: self.tile += 1
                    #if e.key == pygame.K_F2:
                    #    self.map.save()
                    #if e.key == pygame.K_F3:
                    #    try:
                    #        self.load()
                    #    except IOError:
                    #        self.add_message("Could not load level!")
                    #if e.key == pygame.K_KP_PLUS:
                    #    self.map.id += 1
                    #if e.key == pygame.K_KP_MINUS:
                    #    self.map.id -= 1
                elif e.type == pygame.KEYUP:
                    if e.key == pygame.K_LEFT: self.k &= ~1
                    if e.key == pygame.K_RIGHT: self.k &= ~2
                    if e.key == pygame.K_UP: self.k &= ~4
                    if e.key == pygame.K_DOWN: self.k &= ~8
                    if e.key == pygame.K_a: self.k &= ~16
                    if e.key == pygame.K_z or e.key == pygame.K_y:
                        self.k &= ~32
                elif e.type == pygame.MOUSEBUTTONDOWN:
                    if self.editor: self.click(e.button)
                    if e.button == 1 and not self.editor:
                        self.k |= 16
                        if not self.player:
                            self.load()
                    if e.button == 3: self.k |= 32
                elif e.type == pygame.MOUSEBUTTONUP:
                    if e.button == 1: self.k &= ~16
                    if e.button == 3: self.k &= ~32
                elif e.type == pygame.MOUSEMOTION:
                    pass
            
            if self.editor:
                pass
            else:
                self.xm, self.ym = pygame.mouse.get_rel()
                self.update()
            self.render()
        
            t = pygame.time.get_ticks()
            pygame.time.wait(int(next - t))
            t = next
            next = t + 1000 / self.FPS

    def load(self):
        self.map.load()
        self.player = None
        self.sprites = []
        self.level_done = 0
        
        self.time = self.FPS * 10
       
        tiles = self.map.tiles
        for tz in range(1, self.map.h - 1):
            for tx in range(1, self.map.w - 1):
                c, h = tiles[tx + tz * self.map.w]
                if c == PLAYER:
                    s = Allefant(len(self.sprites) + 1, PLAYER,
                        tx + 0.5, tz + 0.5, Tile.d * 0.5)
                    self.sprites.append(s)
                    self.player = s
                if c == TACHYON:
                    s = Tachyon(len(self.sprites) + 1, TACHYON,
                        tx + 0.5, tz + 0.5, Tile.d * 0.5)
                    self.sprites.append(s)
                    self.time += self.FPS * 10

        self.add_message("Warped to Stage %d!" % self.map.id)

    def click(self, button):
        mx, my = self.mousepos
        x, y = self.map.pixel_to_cell_i(mx, my - 16 * self.height)
        px, py = self.map.cell_to_pixel(x, y)
        if button == 1:
            self.map.set(x, y, self.tile, self.height)
        if button == 2:
            pass
        if button == 3:
            self.tile, self.height = self.map.get(x, y)
        if button == 4:
            if self.height > 0: self.height -= 1
        if button == 5:
            if self.height < 64: self.height += 1

    def update(self):
        if self.player:
            self.player.k = self.k
            self.player.xm = self.xm
            self.player.ym = self.ym

        # Collide shots with tachyons
        n = 0
        m = self.map
        grid = [None] * m.w * m.h
        for sprite in self.sprites:
            sprite.update()
            if sprite.type == TACHYON:
                i = sprite.tx + sprite.tz * m.w
                if grid[i] == None: grid[i] = []
                grid[i].append(sprite)
                n += 1
        for shot in self.sprites:
            if shot.type == SHOT:
                w = m.w
                for off in [-1, 0, 1, w - 1, w, w + 1, -w - 1, -w, -w + 1]:
                    i = off + sprite.tx + sprite.tz * w
                    if not grid[i]: continue
                    for tachyon in grid[i]:
                        d = (tachyon.position - shot.position).norm()
                        if d < 16:
                            tachyon.dead = True
                            shot.dead = True
                            self.s_bling.play(1)

        if n == 0:
            if self.level_done:
                if not self.message:
                    self.map.id += 1
                    self.load()
            else:
                self.add_message("All Tachyons Destroyed!")
                self.level_done = 1

        self.sprites = [x for x in self.sprites if not x.dead]
        if self.player and self.player.dead:
            if self.player.reason == "eaten":
                self.add_message("Large Hadron Collision!")
            elif self.player.reason == "fell":
                self.add_message("Head over Heels!")
            else:
                self.add_message("Lost in Spacetime!")
            self.player = None
        
        if self.message_time:
            self.message_time -= 1
            if self.message_time == 0:
                self.message = ""
        
        if self.time > 0:
            self.time -= 1
            if self.time == 0:
                self.add_message("Black Hole has Formed!")
                if self.player:
                    self.player.dead = True
                    self.player = None

        self.tick += 1

    def add_message(self, text):
        self.message_time = self.FPS * 3
        self.message = text

    def render(self):
        w, h = self.size

        glClearColor(0, 0, 0, 1)
        glClearDepth(1)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        self.map.draw()

        for sprite in self.sprites:
            sprite.draw_3d()

        glEnableClientState(GL_TEXTURE_COORD_ARRAY)
        glDisableClientState(GL_NORMAL_ARRAY)
        glEnable(GL_TEXTURE_2D)
        glDisable(GL_LIGHTING)
        
        glDepthMask(GL_FALSE)
        glColor4f(1, 1, 1, 1)
        for sprite in self.sprites:
            sprite.draw_2d()
        glDepthMask(GL_TRUE)
        
        glDisable(GL_DEPTH_TEST)
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()  
        glOrtho(0, w, h, 0, -1, 1)
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glLoadIdentity()

        mx, my = self.mousepos
        x, y = self.map.pixel_to_cell_i(mx, my - self.height * 16)
        px, py = self.map.cell_to_pixel(x, y)

        if self.editor:
            self.font.text(0, 0, "%d/%d %d/%d" % (self.tile, self.height, x, y))
        else:
            if self.player:
                if self.player.energy:
                    self.font.text(0, 0, "power on", tint = (0, 1, 0, 0.8))
                else:
                    self.font.text(0, 0, "power off", tint = (1, 0, 0, 0.8))
                if self.map.id == 7:
                    self.font.text(self.w / 2, self.h / 4, "You Win!",
                        align = 1)

                if self.map.id == 1:
                    self.font.text(self.w / 2, self.h / 2 + 20,
                        "Cursor keys or mouse to rotate <br> " +
                        "A key or Left button to accelerate <br> " +
                        "Z key or Right button to shoot <br> ",
                        align = 1, wrap = self.w * 0.9)
            self.font.text(self.w / 2, self.h, "Time Left: %d" %
                (self.time // self.FPS),
                align = 1,
                aligny = 2)
        self.font.text(self.w, 0, "Stage %d" % self.map.id, align = 2)
        
        if self.message:
            self.font.text(self.w / 2, self.h / 2,
                self.message, align = 1, aligny = 1)
        
        glPopMatrix()
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_LIGHTING)
        glDisable(GL_TEXTURE_2D)
        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glEnableClientState(GL_NORMAL_ARRAY)

        pygame.display.flip()

  

