from __future__ import division
import OpenGL, pygame
from OpenGL.GL import *
from data import filepath
import main

class Image:
    cache = {}
    def __init__(self, surface, rect = None):
        filename = "none"
        if type(surface) == str or type(surface) == unicode:
            filename = surface
            if filename not in Image.cache:
                #print "loading %s" % filename
                Image.cache[filename] = pygame.image.load(filepath(filename))
            surface = Image.cache[filename]
        self.filename = filename
        data = pygame.image.tostring(surface, "RGBA", True)
        self.w = surface.get_width()
        self.h = surface.get_height()
        self.texture = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, self.texture)
        
        def pot(x):
            y = 1
            while y < x: y *= 2
            return y

        self.texw = pot(self.w)
        self.texh = pot(self.h)
        
        if rect:
            x, y, w, h = rect
            y = self.h - y - h
            data2 = ""
            for j in range(h):
                p = x * 4 + self.w * (y + j) * 4
                data2 += data[p : p + w * 4]
            self.w = w
            self.h = h
            data = data2
        
        if self.texw != self.w or self.texh != self.h:
            padw = (self.texw - self.w) * 4
            data2 = ""
            p = 0
            for y in range(self.texh):
                if y < self.h:
                    data2 += data[p:p + self.w * 4] + "\0" * padw
                    p += self.w * 4
                else:
                    data2 += "\0" * self.texw
            data = data2

        #print "texture %d[%dx%d] for %s(%s)" % (self.texture,
        #    self.w, self.h,
        #    filename, str(rect))
        glTexImage2D(GL_TEXTURE_2D, 0, 4, self.texw, self.texh, 0, GL_RGBA,
            GL_UNSIGNED_BYTE, data)
        glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
        glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)

    def draw(self, x, y, tint = (1, 1, 1, 1)):
        glBindTexture(GL_TEXTURE_2D, self.texture)
        glColor4f(tint[0], tint[1], tint[2], tint[3])
        tw, th = self.w / self.texw, self.h / self.texh

        v = [(x, y), (x, y + self.h), (x + self.w, y), (x + self.w, y + self.h)]
        t = [(0, th), (0, 0), (tw, th), (tw, 0)]
        glTexCoordPointerf(t)
        glVertexPointerf(v)
        glDrawArrays(GL_TRIANGLE_STRIP, 0, 4)
    
    def draw_3d(self, x, y, z):
        glBindTexture(GL_TEXTURE_2D, self.texture)
        tw, th = self.w / self.texw, self.h / self.texh
        s = self.w * 8 ** -0.5
        s2 = -self.h * 32 ** -0.5
        t = self.h * 0.25 * 3 ** 0.5

        v = [(x - s + s2, y + t, z + s + s2),
            (x - s - s2, y - t, z + s - s2),
            (x + s + s2, y + t, z - s + s2),
            (x + s - s2, y - t, z - s - s2)]
        t = [(0, th), (0, 0), (tw, th), (tw, 0)]
        glTexCoordPointerf(t)
        glVertexPointerf(v)
        glDrawArrays(GL_TRIANGLE_STRIP, 0, 4)
    
    def draw_3d2(self, x, y, z):
        glBindTexture(GL_TEXTURE_2D, self.texture)
        tw, th = self.w / self.texw, self.h / self.texh
        w = self.w * 0.5
        h = self.h * 0.5
        v = [(x - w, y, z - h),
            (x - w, y, z + h),
            (x + w, y, z - h),
            (x + w, y, z + h)]
        t = [(0, th), (0, 0), (tw, th), (tw, 0)]
        glTexCoordPointerf(t)
        glVertexPointerf(v)
        glDrawArrays(GL_TRIANGLE_STRIP, 0, 4)

    @staticmethod
    def from_string(string, font, color = (1, 1, 1, 1)):
        r, g, b, a = color
        r *= 255
        g *= 255
        b *= 255
        a *= 255
        return Image(font.render(string, True, (r, g, b, a)))

    def __del__(self):
        if self.texture and not main.over: glDeleteTextures([self.texture])