from __future__ import division
import game
from math import *
from tile import Tile
from data import filepath

import OpenGL
from OpenGL.GL import *

PLAYER = 1
TACHYON = 2
RAMPR = 4
TILE = 5
RAMPL = 8

class Map:
    def __init__(self):
        self.h = 18
        self.w = 18
        self.tiles = [(0, 0)] * self.w * self.h

        self.x = game.game.w / 2
        self.y = -32
        
        self.id = 0
        
        self.dl = 0
        self.rebuild = False

    def cell_to_pixel(self, cx, cy):
        return self.x + cx * 32 - cy * 32, self.y + cx * 16 + cy * 16

    def pixel_to_cell(self, px, py):
        px -= self.x
        py -= self.y
        return (px + py * 2) / 64, (py * 2 - px) / 64
        
    def pixel_to_cell_i(self, px, py):
        x, y = self.pixel_to_cell(px, py)
        return int(floor(x)), int(floor(y))
    
    def xyz_to_xy(self, x, y, z):
        s = 2 ** -0.5
        s2 = s / 2
        t = 3 ** 0.5 / 2
        px = self.x + x * s - z * s
        py = self.y + x * s2 - y * t + z * s2
        return px, py

    def get(self, x, y):
        return self.tiles[x + y * self.w]

    def getz(self, t, h, x, y):
        if t == PLAYER: return h * Tile.d
        if t == TACHYON: return h * Tile.d
        if t == RAMPR: return h * Tile.d + x * Tile.d
        if t == TILE: return h * Tile.d
        if t == RAMPL: return h * Tile.d + y * Tile.d
        return 1024

    def height_above_ground(self, x, z):
        tx = int(x // Tile.w)
        tz = int(z // Tile.h)
        c, h = self.tiles[tx + tz * self.w]
        return -self.getz(c, h, x / Tile.w - tx, z / Tile.h - tz)
    
    def tile_and_height_above_ground(self, x, z):
        tx = int(x // Tile.w)
        tz = int(z // Tile.h)
        c, h = self.tiles[tx + tz * self.w]
        return c, -self.getz(c, h, x / Tile.w - tx, z / Tile.h - tz)

    def set(self, x, y, t, h):
        x, y = int(floor(x)), int(floor(y))
        if x < 0 or y < 0 or x >= self.w or y >= self.h: return
        self.tiles[x + y * self.w] = t, h
        self.rebuild = True
    
    def save(self):
        f = file(filepath("map%d" % self.id), "wb")
        f.write(repr(self.tiles))
    
    def load(self):
        f = file(filepath("map%d" % self.id), "rb")
        self.tiles = eval(f.read())
        self.rebuild = True
        #for i in range(16):
        #    self.tiles[(i + 1) * self.w + 1:(i + 1) * self.w + self.w - 1] =\
        #        t[i * 16:i * 16 + 16]

    def draw(self):

        if self.rebuild:
            if self.dl != 0:
                glDeleteLists(self.dl, 1)
                self.dl = 0
            self.rebuild = False
        if self.dl == 0:
            self.dl = glGenLists(1)
            glNewList(self.dl, GL_COMPILE_AND_EXECUTE)

            s = 32 * sqrt(2)
            height = 16 * 2 / sqrt(3)

            for ty in range(1, self.h - 1):
                for tx in range(1, self.w - 1):
                    c, h = self.tiles[tx + ty * self.w]
                    
                    x, y, z = tx * s, h * -height - height, ty * s

                    if c:
                        glColor4f(.5, .5, .5, 1)
                        if c == RAMPR:
                            v, n = Tile.rampr(x, y, z, s, height, s)
                        elif c == TILE:
                            v, n = Tile.tile(x, y, z, s, height, s)
                        elif c == RAMPL:
                            v, n = Tile.rampl(x, y, z, s, height, s)
                        else:
                            if game.game.editor: glColor4f(.5, 1, .5, 1)
                            v, n = Tile.tile(x, y, z, s, height, s)


                        glNormalPointerf(n)
                        glVertexPointerf(v)
                        glDrawArrays(GL_QUADS, 0, 4 * 6)

            glEndList()
        else:
            glCallList(self.dl)

