#encoding: utf8
from OpenGL.GL import *
import exceptions, struct
from data import filepath

class Mesh:
    def __init__(self):
        self.vertices = []
        self.triangles = []
        self.materials = []
        self.display_list = None

    def render(self):
        if self.display_list == None:
            self.display_list = glGenLists(1)
            glNewList(self.display_list, GL_COMPILE_AND_EXECUTE)
            glBegin(GL_TRIANGLES)
            for t in self.triangles:
                r, g, b, a = self.materials[t.mat]
                glColor4f(r, g, b, a)
                v1 = self.vertices[t.v1]
                v2 = self.vertices[t.v2]
                v3 = self.vertices[t.v3]
                glNormal3f(v1.nx, v1.ny, v1.nz)
                glVertex3f(v1.x, v1.y, v1.z)
                glNormal3f(v2.nx, v2.ny, v2.nz)
                glVertex3f(v2.x, v2.y, v2.z)
                glNormal3f(v3.nx, v3.ny, v3.nz)
                glVertex3f(v3.x, v3.y, v3.z)
            glEnd()
            glEndList()
        else:
            glCallList(self.display_list)

class Vertex:
    def __init__(self, x, y, z, nx, ny, nz):
        self.x = x
        self.y = y
        self.z = z
        self.nx = nx
        self.ny = ny
        self.nz = nz

class Triangle:
    def __init__(self, v1, v2, v3, mat, c1, c2, c3):
        self.v1 = v1
        self.v2 = v2
        self.v3 = v3
        self.mat =mat
        self.c1 = c1
        self.c2 = c2
        self.c3 = c3

def read_str(f):
    str = ""
    for i in range(32):
        c = f.read(1)
        if ord(c) >= 32: str += c
    return str

def getf(f): return struct.unpack("<f", f.read(4))[0]
def geti(f): return struct.unpack("<i", f.read(4))[0]
def getb(f): return f.read(1)

def load_meshes(filename, rescale = None):
    n = 0
    f = file(filepath(filename))
    num_meshes = geti(f)
    meshes = {}
    for mi in range(num_meshes):
        name = read_str(f)
        mesh = Mesh()
        meshes[name] = mesh
        #print "mesh", name

        vn = geti(f)
        for i in range(vn):
            x = getf(f)
            y = getf(f)
            z = getf(f)
            if rescale:
                x, y, z = rescale(x, y, z)
            nx = getf(f)
            ny = getf(f)
            nz = getf(f)
            v = Vertex(x, y, z, nx, ny, nz)
            mesh.vertices.append(v)
        
        mn = geti(f)
        for i in range(mn):
            r = getf(f)
            g = getf(f)
            b = getf(f)
            a = getf(f)
            mesh.materials.append((r, g, b, a))

        tn = geti(f)
        for i in range(tn):
            v1 = geti(f)
            v2 = geti(f)
            v3 = geti(f)
            mat = geti(f)
            c1 = [getb(f) for x in range(4)]
            c2 = [getb(f) for x in range(4)]
            c3 = [getb(f) for x in range(4)]
            t = Triangle(v1, v2, v3, mat, c1, c2, c3)
            mesh.triangles.append(t)

    num_objects = geti(f)
    for oi in range(num_objects):
        name = read_str(f)
        #print "object", name
        meshnum = geti(f)
        location = [getf(f) for x in range(3)]
        rotation = [getf(f) for x in range(3)]

    return meshes
