from __future__ import division
import game
from math import *
from body import Body
from tile import Tile
from vector import Vector
from sprite import Sprite
import random

import OpenGL
from OpenGL.GL import *

class Tachyon(Sprite):
    def __init__(self, id, type, cx, cz, above_ground):
        Sprite.__init__(self, id, type, cx, cz, above_ground)

    def update(self):
        def r():
            return (random.random() - 0.5) * 0.04
        self.momentum += Vector(r(), r(), r())
        self.position += self.momentum
        
        if self.momentum.norm() > 0.2:
            self.momentum = self.momentum.normalize() * 0.2
        
        x, y, z = self.position
        
        self.tx = int(x // Tile.w)
        self.tz = int(z // Tile.h)
        
        c, h = self.map.tile_and_height_above_ground(x, z)
        if c == 0:
            self.position -= self.momentum
            self.momentum = -self.momentum
        else:
            if self.position.y < h + 8:
                self.position.y = h + 8
            if self.position.y > h + Tile.d * 3:
                self.position.y = h + Tile.d * 3
                self.momentum = -self.momentum
        
        p = game.game.player
        if p:
            pos = p.position - p.b * 10 + p.u * 10
            d = (pos - self.position).norm()
            if d < 20:
                p.dead = True
                p.reason = "eaten"
                game.game.s_gone.play(1)
           
    def draw_2d(self):
        x, y, z = self.position.xyz()
        pic = game.game.tachyon[(game.game.tick // 2) % 16]
        h = self.map.height_above_ground(x, z)
        glColor4f(0, 0, 0, 0.5)
        pic.draw_3d2(x, h + 4, z)
        glColor4f(1, 1, 1, 1)
        pic.draw_3d(x, y, z)
       