from __future__ import division
from image import Image
import OpenGL, pygame
from OpenGL.GL import *
from data import filepath

import game

class Font(pygame.font.Font):
    def __init__(self, name, size):
        pygame.font.Font.__init__(self, filepath(name), size)
    def text(self, x, y, string, wrap = 0, align = 0,
        tint = (1, 1, 1, 1), aligny = False):
        Text(self, string, wrap).render(x, y, align, tint,
            aligny = aligny)

class Text:
    cache = {}
    max_entries = 100

    def __init__(self, font, text, wrap):
        self.font = font

        got = Text.cache.get((font, text, wrap), None)
        if got:
            self.lines, tick = got
            Text.cache[(font, text, wrap)] = (self.lines, game.game.tick)
            return

        words = text.split()
        lines = [""]
        wX, hX = font.size("X")
        y = 0
        for word in words:
            if word == "<p>":
                lines.append("")
                lines.append("")
                y += hX * 2
                continue
            if word == "<br>":
                lines.append("")
                y += hX
                continue
            newline = lines[-1]
            if newline: newline += " "
            pw, ph = font.size(newline)
            newline += word
            lw, lh = font.size(newline)
            if wrap == 0 or lw <= wrap:
                lines[-1] = newline
                w, h = font.size(word)
            else:
                lines.append(word)
                w, h = font.size(word)
                y += hX

        self.lines = [
            Image.from_string(x.replace("_", " "), self.font) for x in lines]

        t = game.game.tick
        Text.cache[(font, text, wrap)] = (self.lines, t)
        if len(Text.cache) > Text.max_entries:
            newcache = {}
            for k, v in Text.cache.iteritems():
                if t - v[1] <= 1:
                    newcache[k] = v
            Text.cache = newcache

    def render(self, x, y, align = 0,
        tint = (1, 1, 1, 1), backdrop = (0, 0, 0, 0), aligny = 0):
        minx = x
        maxx = x
        miny = y
        maxy = y
        h = 0
        for line in self.lines:
            if align == 1:
                l = x - line.w / 2
                r = x + line.w / 2
            elif align == 0:
                l = x
                r = x + line.w
            else:
                l = x - line.w
                r = x
            if l < minx: minx = l
            if r > maxx: maxx = r
            h += line.h
        if aligny == 1:
            y -= h / 2
            miny -= h / 2
            maxy += h / 2
        elif aligny == 2:
            y -= h
            miny -= h
        else:
            maxy += h
        
        if backdrop[3] > 0:
            minx -= 16
            maxx += 16
            miny -= 16
            maxy += 16
            glColor4f(*backdrop)
            glDisable(GL_TEXTURE_2D)
            
            v = [(minx, miny), (minx, maxy), (maxx, miny), (maxx, maxy)]
            glVertexPointerf(v)
            glDrawArrays(GL_TRIANGLE_STRIP, 0, 4)
            glEnsable(GL_TEXTURE_2D)

        for line in self.lines:
            if align == 1: line.draw(x - line.w / 2, y, tint = tint)
            elif align == 2: line.draw(x - line.w, y, tint = tint)
            else: line.draw(x, y, tint = tint)
            y += line.h
        return x, y
